%% Newton-Raphson-Verfahren
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut f�r Tecnische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%
clear all

% function
syms x
f(x) = x^4 - 1;
df(x) = diff(f,x);
f = matlabFunction(f);
df = matlabFunction(df);

roots = 4; % number of zero points

%% Graphics settings, RGB colors
red     = [255,  0 ,  0 ];
green   = [ 0 , 255,  0 ];
blue    = [ 0 ,  0 , 255];
yellow  = [255, 255,  0 ];
white   = [255, 255, 255];

stepDiv = 20; % divide color shade

%% Iteration
tol = eps; % tolerance, e.g. reqired accuracy
maxStep = 2e2; % allowed steps

% define grid
n = 100;
xReal = linspace(-1.5,1.5,n);
xImag = linspace(1.5,-1.5,n);
nReal = numel(xReal);
nImag = numel(xImag);

% init results storage
resStep     = NaN(nReal,nImag);
resXtraj    = cell(nReal,nImag);
resColor    = NaN(nReal,nImag,3);
resXn       = NaN(nReal,nImag);

% loop over grid
for kImag = 1:nImag
    for kReal = 1:nReal
        
        x0 = xReal(kReal) + 1i * xImag(kImag); % initial guess

        [xn,step,Xtraj] = newtonRaphsonIteration(x0,f,df,tol,maxStep);
        
        resStep(kReal,kImag)     = step;
%         resXtraj{kReal,kImag}    = Xtraj;        
        resXn(kReal,kImag)       = xn;
        
    end % end kReal
end % end kImag


shadeFactors = zeros(roots,1); 
colorClassification = zeros(roots,1); 

for r = 1:roots
    shadeFactors(r) = 1-(r-1)/roots;    
    colorClassification(r) = stepDiv/roots*r;
end

for kImag = 1:nImag
    for kReal = 1:nReal 

        xn = resXn(kReal,kImag); 
        step = resStep(kReal,kImag);
        

        % chose shade factor
        colorStep = mod(step,colorClassification(end));
        idx = find(colorStep<=colorClassification);
        sFactor = shadeFactors(idx(1));
        
        % color
        if     abs(xn - 1)  < tol; color = blue;
        elseif abs(xn + 1)  < tol; color = green;
        elseif abs(xn - 1i) < tol; color = yellow;
        elseif abs(xn + 1i) < tol; color = red;
        else   color = white; sFactor = 1; % in case of no convergence
        end
                
        color = color*sFactor;
        resColor(kImag,kReal,:)    = color;

    end
end

% Achtung: Linux Befehle!
imwrite(uint8(resColor),'test.bmp');

system('konqueror test.bmp &');

save('z4-1_fractal.mat');


 